package com.mvplugin.core.api;

import com.dumptruckman.minecraft.pluginbase.entity.BasePlayer;
import com.mvplugin.core.minecraft.Entity;

import java.util.List;

/**
 * Used to remove animals from worlds that don't belong there.
 */
public interface WorldPurger {
    /**
     * Synchronizes the given worlds with their settings.
     *
     * @param worlds A list of {@link com.mvplugin.core.api.MultiverseWorld}
     */
    void purgeWorlds(List<MultiverseWorld> worlds);

    void purgeWorld(MultiverseWorld world);

    /**
     * Clear all animals/monsters that do not belong to a world according to the config.
     *
     * @param mvworld The {@link MultiverseWorld}.
     * @param thingsToKill A {@link java.util.List} of animals/monsters to be killed.
     * @param negateAnimals Whether the monsters in the list should be negated.
     * @param negateMonsters Whether the animals in the list should be negated.
     */
    void purgeWorld(MultiverseWorld mvworld, List<String> thingsToKill, boolean negateAnimals,
                    boolean negateMonsters);

    /**
     * Clear all animals/monsters that do not belong to a world according to the config.
     *
     * @param mvworld The {@link MultiverseWorld}.
     * @param thingsToKill A {@link java.util.List} of animals/monsters to be killed.
     * @param negateAnimals Whether the monsters in the list should be negated.
     * @param negateMonsters Whether the animals in the list should be negated.
     * @param sender The {@link BasePlayer} that initiated the action. He will/should be notified.
     */
    void purgeWorld(MultiverseWorld mvworld, List<String> thingsToKill, boolean negateAnimals,
                    boolean negateMonsters, BasePlayer sender);

    /**
     * Determines whether the specified creature should be killed.
     *
     * @param e The creature.
     * @param thingsToKill A {@link java.util.List} of animals/monsters to be killed.
     * @param negateAnimals Whether the monsters in the list should be negated.
     * @param negateMonsters Whether the animals in the list should be negated.
     * @return {@code true} if the creature should be killed, otherwise {@code false}.
     */
    boolean shouldWeKillThisCreature(Entity e, List<String> thingsToKill, boolean negateAnimals, boolean negateMonsters);

    /**
     * Determines whether the specified creature should be killed and automatically reads the params from a world object.
     *
     * @param w The world.
     * @param e The creature.
     * @return {@code true} if the creature should be killed, otherwise {@code false}.
     */
    boolean shouldWeKillThisCreature(MultiverseWorld w, Entity e);
}
